package Testrig 
  
  model Wegvorgabe 
    Modelica.Blocks.Interfaces.RealInput u;                          // Laden des Modelica Konnektors RealInput
    Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a;   // Laden des Modelica Konnektors Flange_a
  equation 
    flange_a.s = u;
  end Wegvorgabe;
  

  model Federbein
    parameter Real c(unit="N/m")=30000;
    parameter Real d(unit="Ns/m")=3000;
    Real F(unit="N");
    Real z(unit="m");
    Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a;  // Laden des Modelica Konnektors Flange_a
    Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b;  // Laden des Modelica Konnektors Flange_b
  equation
    z=flange_a.s-flange_b.s;
    F=z*c+d*der(z);
    flange_a.f=F;
    flange_b.f=-F;
  end Federbein;
  

  model Luftfeder

    parameter Real pi = Modelica.Constants.pi;
    parameter Modelica.SIunits.Diameter d = 1e-1 "Kolbendurchmesser";      // Kolbendurchmesser [m]
    parameter Modelica.SIunits.Length h = 2e-1 "Federraumhhe";            // Federraumhhe [m]
    parameter Modelica.SIunits.Area A = pi/4*d^2 "Kolbenflche";           // Kolbenflche [m]
    parameter Modelica.SIunits.Area S = 2*A+pi*d*h "Gehuseflche";        // Gehuseflche [m]
    parameter Modelica.SIunits.Volume V0 = A*h "Anfangsvlumen";            // Volumen der Luftfeder [m]

    //parameter Real amp = 0.01; // Wegamplitude in N
    //parameter Real freq = 0.01; // Frequenz der Weganregung in Hz
    parameter Real gamma = 1.4;                                            // Adiabatenexponent [-]
    parameter Real k(unit="W/m/K") = 15 "Wrmedurchgangszahl";            // Wrmedurchgangszahl [W/m/K]
    parameter Real R(unit="J/kg/K") = 287 "Gaskonstante";                  // Gaskonstante von Luft [J/kg/K]
    parameter Real T0(unit="K") = 296 "Umgebungstemperatur";               // Umgebungstemperatur [K]
    parameter Real pu(unit="Pa")= 1e5 "Umgebungsdruck";                    // Umgebungstemperatur [K]

    Real rho(start=9.4171, unit="kg/m"); // Dichte in der Luftfeder
    Real z(unit="m"); // Kolbenweg
    Real p(start=8e5, unit="Pa"); // Druck in der Luftfeder
    Real T(unit="K"); // Temperatur in der Luftfeder
    Real F(unit="N"); // Kolbenkraft
    
    Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a;  // Laden des Modelica Konnektors Flange_a
    Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b;  // Laden des Modelica Konnektors Flange_b
  
  equation
    z = flange_b.s-flange_a.s;
    flange_a.f = F;
    flange_b.f = -F;
    p = rho*R*T;                                                // ideale Gasgleichung
    der(rho) = A*rho*der(z)/(V0-A*z);                           // Kontinuittsgleichung
    der(p) = (A*der(z)*gamma*p-(gamma-1)*S*k*(T-T0))/(V0-z*A);  // Energiegleichung
    F = (p-pu)*A;
  
  end Luftfeder;
  
  model Systemaufbau          // In diesem Modell wird das gesamte System zusammengesetzt     
    Wegvorgabe Wegvorgabe1;   // Einfgen des Modells Wegvorgabe. Dieses trgt innerhalb des Modells Systemaufbau die Bezeichnung Wegvorgabe1
    Luftfeder Luftfeder1;     // Hier das Federbein einfgen, analog wie bei Block Wegvorgabe
    Modelica.Mechanics.Translational.Fixed Fixed1;                // Einfgen des Modelica Blocks Fixed.
    Modelica.Blocks.Sources.Sine Sine1(amplitude=0.01, freqHz=0.5); // Einfgen des Modelica Blocks Sine
  equation 
    connect(Sine1.y,Wegvorgabe1.u);       // Hier wird der Block Sine1 mit dem Block Wegvorgabe1 verbunden
    connect(Wegvorgabe1.flange_a, Luftfeder1.flange_b);  // Hier den Block Wegvorgabe mit Federbein verbinden
    connect(Luftfeder1.flange_a ,Fixed1.flange_b);       // Hier den Block Federbein mit Fixed verbinden
  end Systemaufbau;
  
end Testrig;
