function omeinlesen_einzeln(modelname,index0)  
%-----------------------------------------------------------------------------------------
%Funktion zum Einlesen eines OpenModelica Result Files (MODELNAMME_res.plt) und Sichern 
%aller berechneten Variablen. Je nach Wahl des Eingabeparameters index0, werden die
%Vaiablen entweder in den Matlab Workspace, in den Workspace der aufrufenden Funktion oder
%in ein Matfile geschrieben.
%
%ACHTUNG! Abhngig von der Komplexitt des Modells, der Simulationszeit und der fr die 
%         Simulation verwendeten Schrittweite, kann es in Extremfllen vorkommen, dass der 
%         Speicher von Matlab nicht ausreicht, um die Daten einzulesen. In diesem Fall 
%         sollte entweder die Schrittweite vergrert oder die Simulationszeit verkrzt 
%         werden. Sollte dies nicht ausreichen, muss das Resultfile des Modelica Modells 
%         von Hand geffnet und die interessierenden Daten fr das Einlesen in einem 
%         zweiten Resultfile gespeichert werden.
%
%SYNTAX:    omeinlesen_einzeln(modelname,index0)
%
%mit:       modelname = NAME DES MODELLS bzw. NAME DES PACKAGES.NAME DES MODELLS (falls
%                       das Modell innerhalb eines Packagees liegt:
%                       z.B. 'Testrig.Systemaufbau'
%           index0    = 0 (Variablen in den Matlab Workspace schreiben) / 1 (Variablen in 
%                       den Workspace der aufrufenden Funktion schreiben) / 2 (Variablen 
%                       in den Workspace der aufrufenden Funktion und den Matlab Workspace 
%                       schreiben / 3 (Variablen in einem Matfile speichern)
%
%(c) Schaad,  Februar 2008
%(m) Hamadeh, April   2008
%-----------------------------------------------------------------------------------------
%%
[daten datasetindexlaenge] = omlesennachdaten(modelname);

% Abfrage nach dem Speicherort der Variablen bei entsprechendem Funktionsaufruf (index0 = 2)
if index0 == 3
    pfad = cd;
    [datei, pfad] = uiputfile('*.mat','Speichern der Simulationsergebnisse');
end

for n = 1:datasetindexlaenge+1;
    % Entfernen der dummy Variablen, der von Modelica erzeugten Ableitungen der Variablen des Modells und der Variablen die 0 sind
    if isempty(strfind(daten(1).name,'$dummy')) & isempty(strfind(daten(1).name,'der(')) & (max(daten(1).data)~=0 | min(daten(1).data)~=0)
        % Hier werden die Sonderzeichen durch Unterstriche ersetzt, EVTL. ZU ERWEITERN !!!
        if (isempty(strfind(daten(1).name,'(')) ~= 1) || (isempty(strfind(daten(1).name,'[')) ~= 1) || (isempty(strfind(daten(1).name,',')) ~= 1)
            daten(1).name(strfind(daten(1).name,'('))='';
            daten(1).name(strfind(daten(1).name,')'))='';
            if isempty(strfind(daten(1).name,'[')) ~= 1
                index1 = strfind(daten(1).name,'[');
                daten(1).name = [daten(1).name(1:index1-1),'_k',daten(1).name(index1+1:end)];
            end
            daten(1).name(strfind(daten(1).name,']'))='';
            daten(1).name(strfind(daten(1).name,','))='';
        end;

        switch index0
            case 0
                % Variablen in den Matlab Workspace schreiben
                assignin('base','daten',daten(1));
                evalin('base',[daten(1).name,'=daten(1).data;']);
                evalin('base','clear daten');
            case 1
                % Variablen in den Workspace der aufrufenden Funktion schreiben
                assignin('caller','daten',daten(1));
                evalin('caller',[daten(1).name,'=daten(1).data;']);
                evalin('caller','clear daten');
            case 2
                % Variablen in den Workspace der aufrufenden Funktion schreiben
                assignin('caller','daten',daten(1));
                evalin('caller',[daten(1).name,'=daten(1).data;']);
                evalin('caller','clear daten');
                
                % Variablen in den Matlab Workspace schreiben 
                assignin('base','daten',daten(1));
                evalin('base',[daten(1).name,'=daten(1).data;']);
                evalin('base','clear daten');
            case 3
                % Variablen in einer Datei speichern
                % Zuweisen der Variablennamen zu den entsprechenden Werten (z.B. time = Zeitvektor)
                eval([daten(1).name,'=daten(1).data;'])
                S{n} = daten(1).name;
        end
    end
    daten(1) = [];
end
%% Speicherung der einzelnen Variablen bei entsprechendem Funktionsaufruf (index0 = 2)
if index0 == 3
    for n = 1:length(S)
        index2(n) = ~isempty(S{n});
    end
    index3 = find(index2==1);
    for n = 1:length(index3)
        index4 = min(strfind(S{index3(n)},'.'));
        if index4
            Sneu{n} = S{index3(n)}(1:index4-1);
        else
            Sneu{n} = S{index3(n)};
        end
    end
    for n = 1:length(Sneu)
        if n == 1
            eval('save(fullfile(pfad,datei),eval(''Sneu{n}''))')
        else
            eval('save(fullfile(pfad,datei),eval(''Sneu{n}''), ''-append'')')
        end
    end
end
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                       Subfunktionen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%*****************************************************************************************
%Funktion zum paketweisen Auslesen des Resultfiles Speichern des Inhalts in dem 
%character array I und variablenweisen Konvertierung der Daten in double precission Format
%*****************************************************************************************

function [daten, datasetindexlaenge] = omlesennachdaten(modelname)

lf = char(10); % Das Ascii Zeichen 10 ist der linefeed Character bzw. das Zeichen fr einen Zeilenumbruch (er zhlt selbst auch als 1 Zeichen)
Resultfile = [modelname,'_res.plt'];
fid = fopen(Resultfile, 'r');

stringlaenge = 50000000;  % Choose as big as possible to improve speed!!!
offset = 0;
datasetindexlaenge = 0;
daten.name = [];
while  ~feof(fid)
    fseek(fid,-offset-2,0);
    I = fread(fid,stringlaenge,'*char')'; % Auslesen des Resultfiles in das Character Array I (durchlaufend nummeriert)
    Read_MB = ftell(fid)/1024/1024;    % Anzeige des Auslesestatus des Resultfiles
    m = 1;
    anz1 = length(I);
    for n = 2:anz1
        if ((I(n-1)==lf) && (I(n)=='D')); % Wenn ein Zeilenumbruch war und ein neuer Datenblock anfngt
            datasetindex(m) = n; % Finden der Indices des Beginns eines neuen Datenblocks
            m = m+1;
        end
    end

    datasetindex(end+1) = anz1; % Finden des Indices des letzten Zeilenumbruchs

    for n = 1:length(datasetindex)-1
        [temp1,temp2] = strtok(I(datasetindex(n)+9:datasetindex(n+1)-2),lf); % Variablenweises Auslesen der Daten in die Variable temp2
        %temp1 = erste Zeichenfolge in dem angegebenen String, die mit einem Leerzeichen endet
        %temp2 = der Rest des angegebenen Strings (hier: alle Daten)
        linux = isunix;
        switch linux
            case 1
                daten(end+1).name = temp1;          % Ermittlung der Variablennamen
            case 0
                daten(end+1).name = temp1(1:end-1); % Ermittlung der Variablennamen und Entfernung des Zeilenumbruchs
        end
        % Variablenweises Auslesen und Schreiben der Daten in ein cell array.
        c = textscan(temp2,'%f%s%f'); % Zahlenstrings werden dabei automatisch in double precission formatiert, normale Strings werden beibehalten
        daten(end).data = c{3};% c hat 3 Zellen: 1. Zelle = Zeitvektor, 2. Zelle= Zwischenstrings,
            %                                    3. Zelle = Vektor mit den Werten der Variable
    end     
    offset = datasetindex(end)-datasetindex(end-1);
    datasetindexlaenge = datasetindexlaenge+length(datasetindex)-1;
    clear I datasetindex
end

fclose(fid); 
